package framework.servlet;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;

import framework.form.BaseForm;
import framework.service.ServiceParameter;
import framework.taglib.ErrorsTag;
import framework.taglib.InfosTag;

public class BaseAction extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public static final String SERVLET_PATH    = "/serv/";
	public static final String SERVICE_PACKAGE = "service";
	public static final String SERVICE_SUFFIX  = "Service";
	public static final String SERVICE_METHOD  = "executeMain";
	public static final String ERROR_PAGE      = "error.html";
	public static final String VIEW_ATTRIBUTE_NAME = "form";
	
	public void doGet(HttpServletRequest request, HttpServletResponse response)
	throws IOException, ServletException {
		process(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException {
		process(request, response);
	}
	
	protected void process(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException {

		// T[rXNX擾
		String serviceClass = getServiceClass(request);

		// ̓f[^̎擾
		BaseForm formData = getFormData(serviceClass, request);

		// T[rXւ̈쐬
		ServiceParameter param = new ServiceParameter();
		param.setFormData(formData);
		param.setRequest(request);
		param.setResponse(response);
//		param.setDbCon(con);

		// T[rXNX̎s
		String nextJsp = executeServiceClass(serviceClass, param);

		// T[rX̏o̓f[^Zbg
		setFormData(request, param);

		// GbZ[WA탁bZ[WZbg
		setMessageList(request, param.getErrorList(), param.getInfoList());

		// ʑJ
		RequestDispatcher rd = getServletContext().getRequestDispatcher(nextJsp);
		if(rd == null){
			response.sendError(500, ERROR_PAGE);
			return;
		} else{
			rd.forward(request, response);
			return;
		}
	}

	protected String getServiceClass(HttpServletRequest request){
		String uri = request.getRequestURI().replaceAll(request.getContextPath()+SERVLET_PATH, "");
		String servicePackage  = uri.toLowerCase();
		String serviceClass = null;
		if (uri.lastIndexOf(".")>0){
			serviceClass    = SERVICE_PACKAGE + "." + servicePackage + "." + uri.substring(uri.lastIndexOf(".")+1) + SERVICE_SUFFIX;
		} else {
			serviceClass    = SERVICE_PACKAGE + "." + uri + SERVICE_SUFFIX;
		}
		return serviceClass;
	}

	protected String executeServiceClass(String serviceClass, ServiceParameter param){
		try {
			Class c = Class.forName(serviceClass);
			Object service = (Object)c.newInstance();
			Class[] cls = new Class[]{ServiceParameter.class};
			Method mthod = Class.forName(serviceClass).getMethod(SERVICE_METHOD, cls);
			Object[] obj = new Object[]{ param };
			return (String)mthod.invoke(service, obj);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		return "";
	}

	protected BaseForm getFormData(String serviceClass , HttpServletRequest request){

		try {
			request.setCharacterEncoding("Windows-31J");
		} catch(Exception e){}

		BaseForm formData = null;

		String prefix = "";
		String suffix = "";

		String formClass = serviceClass.replaceAll("Service", "Form");

		try {
			Class c = Class.forName(formClass);
			formData = (BaseForm)c.newInstance();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			return null;
		}
		
		HashMap properties = new HashMap();
        Enumeration names = request.getParameterNames();
        while(names.hasMoreElements()) 
        {
            String name = (String)names.nextElement();
            String stripped = name;
            if(prefix != null)
            {
                if(!stripped.startsWith(prefix))
                    continue;
                stripped = stripped.substring(prefix.length());
            }
            if(suffix != null){
                if(!stripped.endsWith(suffix))
                    continue;
                stripped = stripped.substring(0, stripped.length() - suffix.length());
            }
            Object parameterValue = null;
            parameterValue = request.getParameterValues(name);
            properties.put(stripped, parameterValue);
        }
		
		try {
			BeanUtils.populate(formData, properties);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}

		return formData;
	}

	protected void setFormData(HttpServletRequest request, ServiceParameter param){

		BaseForm formData = param.getFormData();
		request.setAttribute(VIEW_ATTRIBUTE_NAME, formData);

		HashMap sesObj = formData.getSessionObject();
		if (sesObj!=null){
			Iterator it = sesObj.keySet().iterator();
			while (it.hasNext()){
				String key = (String)it.next();
				request.getSession().setAttribute(key, sesObj.get(key));
			}
		}
	}
	
	protected void setMessageList(HttpServletRequest request
	                           , ArrayList errorList
	                           , ArrayList infoList){
		request.setAttribute(ErrorsTag.REQUEST_ATTRIBUTE_NAME, errorList);
		request.setAttribute(InfosTag.REQUEST_ATTRIBUTE_NAME, infoList);
	}
}
