package framework.taglib;

import java.io.IOException;
import java.util.ArrayList;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.http.HttpServletRequest;

import org.apache.taglibs.standard.lang.jpath.expression.BooleanLiteral;

import framework.service.ServiceMessage;

public class ErrorJscriptTag extends TagSupport {

	public static final String  DEFAULT_COLOR  = "red";
	public static final Boolean DEFAULT_FOCUS  = Boolean.FALSE;
	public static final Boolean DEFAULT_ALERT  = Boolean.FALSE;
	public static final Boolean DEFAULT_JSHEADER = Boolean.TRUE;
	public static final String  DEFAULT_ERRPAGE_ATT  = "dialogWidth=400px; dialogHeight=200px;";

    protected String name;
    protected Boolean focus;
    protected Boolean alert;
    protected String color;
    protected String styleClass;
    protected Boolean jsheader;
    protected String errpage;
    protected String errpageAtt;

    public ErrorJscriptTag(){
    }

	public void release() {
		super.release();
	}

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

	public Boolean getAlert() {
		return (alert != null) ? alert : DEFAULT_ALERT;
	}

	public void setAlert(Boolean alert) {
		this.alert = alert;
	}

	public String getColor() {
		return (color != null) ? color : DEFAULT_COLOR;
	}

	public void setColor(String color) {
		this.color = color;
	}

	public Boolean getFocus() {
		return (focus != null) ? focus : DEFAULT_FOCUS;
	}

	public void setFocus(Boolean focus) {
		this.focus = focus;
	}

	public String getStyleClass() {
        return styleClass != null ? styleClass : "";
	}

	public void setStyleClass(String styleClass) {
		this.styleClass = styleClass;
	}
	
	public Boolean getJsheader() {
        return jsheader != null ? jsheader : DEFAULT_JSHEADER;
	}

	public void setJsheader(Boolean jsheader) {
		this.jsheader = jsheader;
	}

	public String getErrpage() {
		return errpage;
	}

	public void setErrpage(String errpage) {
		this.errpage = errpage;
	}

	public String getErrpageAtt() {
		return errpageAtt != null ? errpageAtt : DEFAULT_ERRPAGE_ATT;
	}

	public void setErrpageAtt(String errpageAtt) {
		this.errpageAtt = errpageAtt;
	}

	public int doStartTag() throws JspException {

		ServletRequest request = super.pageContext.getRequest();

		ArrayList msgList = getMessageList(request);
    	if ((msgList == null)||(msgList.size()==0)){
    		return 1;
    	}    	
    	
    	StringBuffer results = new StringBuffer();
    	
        boolean headerDone = false;

        String fieldName = null;
        String errorMsg  = null;

        for (int i = 0; i < msgList.size(); i++){

    		ServiceMessage error = (ServiceMessage)msgList.get(i);
    		fieldName = (error.getKey() != null) ? error.getKey() : "";
    		errorMsg  = error.getMsg();

    		if (!"".equals(fieldName)){

    			if (!headerDone){
    				addHeader(results, getJsheader(), getName());
    				headerDone = true;
    			}

    			if ((getStyleClass() != null)&&(!"".equals(getStyleClass()))){
        			results.append("		var className = objAll[\""+fieldName+"\"].className;");
        			results.append("		var newClassName = \"" + getStyleClass() + "\";");
        			results.append("		if (className) {");
        			results.append("			newClassName = className + ' ' + \"" + getStyleClass() + "\";");
        			results.append("		}");
    				results.append("		objAll[\""+fieldName+"\"].className = newClassName;");
    			} else {
        			results.append("		objAll[\""+fieldName+"\"].style.color=\""+getColor()+"\";");
    			}

    			results.append("	if ((!focusField)&&(objAll[\""+fieldName+"\"])) {");
//    			results.append("		if (objAll[\""+fieldName+"\"].type == \"text\" ){" );
    			results.append("			focusField = objAll[\""+fieldName+"\"];" );
//    			results.append("		}");
    			results.append("	}");
    		}

    		if(errorMsg != null) {
    			if (!headerDone){
    				addHeader(results, getJsheader(), getName());
    				headerDone = true;
    			}
    			results.append("	errMsg[errMsg.length] = \""+errorMsg+"\";");
    			results.append("	errorList[errorList.length] = { \"key\" : \"" + fieldName + "\", \"msg\" : \"" + errorMsg + "\" };");
    		}
        }

        if ((headerDone)&&(Boolean.TRUE.equals(getAlert()))){
        	String errpageURL = errpage;
        	if (errpage!=null){
        		if (!".".equals(errpage.substring(0,1))){
        			HttpServletRequest req = (HttpServletRequest)super.pageContext.getRequest();
        			errpageURL = req.getContextPath() + errpage;
        		}
        		results.append("	try {");
        		results.append("		url = \"" + errpageURL + "\";");
        		results.append("		var args = new Array();");
        		results.append("		args[0] = errorList;");
        		results.append("		var result = window.showModalDialog( url , args , \"" + getErrpageAtt() + "\" );");
        		results.append("	} catch(e) {");
        	}
			results.append("		var allMsg = errMsg.join(\"\\n\");");
			results.append("		alert(allMsg);");
        	if (errpage!=null){
        		results.append("	}");
        	}
		}

        if ((headerDone)&&(Boolean.TRUE.equals(getFocus()))) {
			results.append("	if (focusField) {");
			results.append("		try {");
			results.append("			focusField.focus();");
			results.append("			focusField.select();");
			results.append("		} catch(e) {}");
			results.append("	}");
		}

        if (headerDone){
        	addFooter(results, getJsheader());
		}

        write(super.pageContext, results.toString());

        return 1;
    }

	protected ArrayList getMessageList(ServletRequest request){
		return (ArrayList)request.getAttribute(ErrorsTag.REQUEST_ATTRIBUTE_NAME);
	}

	private void addHeader(StringBuffer results, Boolean needHeader, String funcName){
		if (Boolean.TRUE.equals(needHeader)){
			results.append("<script type=\"text/javascript\">");
		}
    	results.append("function "+ funcName +"(){");
    	results.append("	var objAll = document.forms[0];");
    	results.append("	var errMsg = new Array();");
    	results.append("	var errorList = new Array();");
    	results.append("	var focusField = false;");
		results.append("	var focusTarget = [\"text\" , \"checkbox\", \"select-one\", \"select-multiple\", \"radio\", \"textarea\" ];");
	}

	private void addFooter(StringBuffer results, Boolean needHeader){
		results.append("}");
		if (Boolean.TRUE.equals(needHeader)){
			results.append("</script>");
		}
	}

	private void write(PageContext pageContext, String text)
		throws JspException {
		JspWriter writer = pageContext.getOut();
		try{
			writer.print(text);
		}
		catch(IOException e){
		}
	}
}
