package framework.taglib;

import java.io.IOException;
import java.util.ArrayList;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

import framework.service.ServiceMessage;

public class ErrorsTag extends TagSupport {

	public static final String REQUEST_ATTRIBUTE_NAME = "framework.errors";
    public static final String DEFAULT_LINE_END = "<br/>";
	
    protected String name;
    protected String prefix;
    protected String suffix;
    protected String styleClass;
    protected String lineEnd;

    public ErrorsTag(){
    }

	public void release() {
		super.release();
	}

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public String getPrefix()
    {
        return prefix != null ? prefix : "";
    }

    public void setPrefix(String prefix)
    {
        this.prefix = prefix;
    }

    public String getSuffix()
    {
        return suffix != null ? suffix : "";
    }

    public void setSuffix(String suffix)
    {
        this.suffix = suffix;
    }
    
	public String getLineEnd() {
		return lineEnd != null ? lineEnd : DEFAULT_LINE_END;
	}

	public void setLineEnd(String lineEnd) {
		this.lineEnd = lineEnd;
	}

	public String getStyleClass() {
        return styleClass != null ? styleClass : "";
	}

	public void setStyleClass(String styleClass) {
		this.styleClass = styleClass;
	}

	public int doStartTag() throws JspException {

		ServletRequest request = super.pageContext.getRequest();

//		pageContext.setAttribute("", "", pageContext.APPLICATION_SCOPE);
//		pageContext.getAttribute("",pageContext.APPLICATION_SCOPE);

		ArrayList msgList = getMessageList(request);
    	if ((msgList == null)||(msgList.size()==0)){
    		return 1;
    	}    	
    	
    	StringBuffer results = new StringBuffer();
        
        boolean prefixPresent = ((getPrefix()==null)||("".equals(getPrefix()))) ? false : true;
        boolean suffixPresent = ((getSuffix()==null)||("".equals(getSuffix()))) ? false : true;
        boolean nameCondition = ((getName()==null)||("".equals(getName()))) ? false : true;

        boolean headerDone = false;

        String message = null;

        for (int i = 0; i < msgList.size(); i++){

    		ServiceMessage error = (ServiceMessage)msgList.get(i);
            message = error.getMsg();

            if(message != null) {
	        	if (nameCondition) {
	        		if (getName().equals(error.getKey())){

	                	if(!headerDone){
	                		if (!"".equals(getStyleClass())){
		                        results.append("<span class=\""+""+"\">");
	                		}
	                        headerDone = true;
	                    }

	                	if(prefixPresent){
	                        results.append(getPrefix());
	                    }

	        			results.append(message);
		    			results.append(getLineEnd());

	                    if(suffixPresent){
	                        results.append(getSuffix());
	        			}
	        		}
	        	} else {

                	if(!headerDone){
                		if (!"".equals(getStyleClass())){
	                        results.append("<span class=\""+getStyleClass()+"\">");
                		}
                        headerDone = true;
                    }

                	if(prefixPresent){
	                    results.append(getPrefix());
	                }
	
	    			results.append(message);
	    			results.append(getLineEnd());
	
	                if(suffixPresent){
	                    results.append(getSuffix());
	    			}
	        	}
        	}
        }

    	if ((headerDone)&&(!"".equals(getStyleClass()))){
    		results.append("</span>");
        }

    	write(super.pageContext, results.toString());

        return 1;
    }

	protected ArrayList getMessageList(ServletRequest request){
		return (ArrayList)request.getAttribute(ErrorsTag.REQUEST_ATTRIBUTE_NAME);
	}

    public void write(PageContext pageContext, String text)
		throws JspException {
		JspWriter writer = pageContext.getOut();
		try{
			writer.print(text);
		}
		catch(IOException e){
		}
	}
}
